//
// Implementent classes to be provided by both the simulator and cluster environments.
//

using System;
using System.Diagnostics;
using MessageRouting;
using SkipNet=Microsoft.Research.Herald.SkipNet;
using Microsoft.Research.Herald.Common;

using Microsoft.Research.Taro;
using Microsoft.Research.Herald.ClusterLib;

namespace Microsoft.Research.Herald.ClusterLib
{
	/// <summary>
	/// Cluster implementation of environment-independent properties of the execution environment
	/// </summary>
	public class ClusterExecutionEnvironment : ExecutionEnvironment
	{
        static ClusterNode clusterNode;

        public ClusterExecutionEnvironment(ClusterNode clusterNode)
        {
            ClusterExecutionEnvironment.clusterNode = clusterNode;
            ClusterLibParameters.Init(clusterNode);
        }

        /// <summary>
        /// Returns execution environment type
        /// </summary>
        public ExecutionEnvironmentType Type()
        {
            return ExecutionEnvironmentType.ClusterEnvironment;
        }

        /// <summary>
        /// Test whether we are the specified node number.
        /// </summary>
        public bool AmNode(int nodeNumber)
        {
            return (ClusterLibParameters.nodeNumber == nodeNumber);
        }

        /// <summary>
        /// Returns the node pointer for the specified node, if we are that node.
        /// </summary>
        public SkipNet.Node GetNode(int nodeNumber)
        {
            Debug.Assert(ClusterLibParameters.nodeNumber == nodeNumber); // Can only get own node pointer on live nodes 
            return clusterNode.node;
        }

        /// <summary>
        /// Total number of nodes to be used in the experiment.
        /// </summary>
        public int NodeCount
        {
            get {return ClusterLibParameters.nodes;}
            set {ClusterLibParameters.nodes = value;}
        }

        /// <summary>
        /// Proceed with queued experiment steps
        /// </summary>
        public void Go()
        {
//            Console.WriteLine("Go()");
        }

        /// <summary>
        /// Run experiment steps for the specified time duration
        /// </summary>
        public void GoFor(int milliseconds)
        {
            if (milliseconds > 1000)
                Console.WriteLine("GoFor({0})...", milliseconds);
            System.Threading.Thread.Sleep(milliseconds);
            if (milliseconds > 1000)
                Console.WriteLine(" GoFor({0}) done.", milliseconds);
        }

        // Gets an address we can use to send messages to a node
        public NodePhysicalAddress NodePhysAddr(int nodeNumber)
        {
            return new TaroAddress(ClusterNodeTable.clusterNodeTable[nodeNumber].NodeAddr());
        }

         // Gets a NodeIdAddressBind for an arbitary node
        public SkipNet.NodeIdAddressBind NodeIdAddrBind(int nodeNumber)
        {
            return new SkipNet.NodeIdAddressBind(NodePhysAddr(nodeNumber), null, null);
        }

        // Must be locked during calls that execute event handlers
        public object OverlayLock 
        {
            get { return TaroLocalEvent.overlayLock; }
        }
	}
}
