//
// Read and manage entries from the cluster node table.
//
// Entries contain the tab-separated data:
//     Logical_Node_Name IP_Address Address_Mask Machine_Name Port_Number
//
// Example entries are:
//     node0 172.31.40.193 255.255.248.0 mbj-laptop 10000
//     node1 172.31.40.193 255.255.248.0 mbj-laptop 10001
//

using System;
using System.Collections;
using System.Diagnostics;
using System.IO;

namespace Microsoft.Research.Herald.ClusterLib
{
    public class ClusterNodeTable 
    {
        const string ClusterNodeTableFile = @"C:\SkipNet\Nodes\ClusterNodeTable.txt";
        public static ClusterNodeTableEntry [] clusterNodeTable = null;

        public static void ReadClusterNodeTable()
        {
            ArrayList tableEntries = new ArrayList();

            if (! File.Exists(ClusterNodeTableFile)) 
            {
                Console.WriteLine();
                Console.WriteLine("File {0} does not exist.", ClusterNodeTableFile);
                Console.WriteLine("You need to create one based on the example in the release's Nodes directory.");
                Environment.Exit(1);
            }
            StreamReader sr = new StreamReader(ClusterNodeTableFile);
            while (true)
            {
                string line = sr.ReadLine();
                if (line == null) break;
                string [] splitLine = line.Split(new Char[] {'\t', ' '}, 5);
                if (splitLine.Length != 5) 
                {
                    Console.WriteLine("ClusterNodeTable line \"{0}\" doesn't contain 5 tab-separated fields -- aborting ReadClusterNodeTable()",
                        line);
                    break;
                }
                ClusterNodeTableEntry entry = new ClusterNodeTableEntry(
                    splitLine[0], splitLine[1], splitLine[2], splitLine[3], splitLine[4]);
                tableEntries.Add(entry);
            }

            clusterNodeTable = new ClusterNodeTableEntry [tableEntries.Count];
            tableEntries.CopyTo(0, clusterNodeTable, 0, tableEntries.Count);
        }
    }
    
    public class ClusterNodeTableEntry
    {
        public string logicalNodeName;
        public string ipAddress;
        public string ipMask;
        public string machineName;
        public string portNumber;

        public ClusterNodeTableEntry(
            string logicalNodeName,
            string ipAddress,
            string ipMask,
            string machineName,
            string portNumber)
        {
            this.logicalNodeName = logicalNodeName;
            this.ipAddress = ipAddress;
            this.ipMask = ipMask;
            this.machineName = machineName;
            this.portNumber = portNumber;
        }

        // Must match algorithm in ClusterNode.cs used to construct clusterNode.nodeAddr
        public string NodeAddr()
        {
            if (ClusterLibParameters.clusterNode.socketMode) 
            {
                return ipAddress + ":" + portNumber;
            } 
            else
            {
                return "soap://" + Environment.MachineName + ":" + portNumber + "/" + logicalNodeName + "/";
            }
        }

        // Must match algorithm in ClusterNode.cs used to construct clusterNode.nodeName
        public string NodeName()
        {
            return "skip://"+ Environment.MachineName + "/" + logicalNodeName + "/";
        }
    }
}
