//
// Parse, store, and print parameters common to the simulator and cluster environments.
//

using System;
using System.Collections;
using SkipNet=Microsoft.Research.Herald.SkipNet;


namespace Microsoft.Research.Herald.Common
{
    public class CommonParameters
    {
        public static int seed = -1;

        // Parse common command line arguments from the specified index,
        // returning the index of the first unparsed parameter.
        // Returns -1 if Usage() needs to be called.
        public static int ParseArgs(string [] args, int starting_at)
        {
            int i = starting_at;
            for (; i < args.Length; i++) 
            {
                if ( (args[i][0] == '-') || (args[i][0] == '/') ) 
                {
                    string opt = args[i].Substring(1);
                    if (opt.Length < 1) break;
                    
                    if ("Seed".StartsWith(opt) && i + 1 < args.Length)
                    {
                        seed = Convert.ToInt32(args[++i]);
                    }
                    else if ("base".StartsWith(opt) && i + 1 < args.Length)
                    {
                        SkipNet.Parameters.b = Convert.ToInt32(args[++i]);
                    }
                    else if (opt.StartsWith("e") || opt.StartsWith("d"))
                    {
                        bool enable = opt.StartsWith("e");
                        string enable_disable_opt = opt.Substring(1);
                        if (enable_disable_opt.Length < 2) break;       // Require at least two chars of option name

                        if ("dense".StartsWith(enable_disable_opt))
                            SkipNet.Parameters.denseRTable = enable;
                        else if ("ptable".StartsWith(enable_disable_opt))
                            SkipNet.Parameters.proximityTableEnabled = enable;
                        else if ("ctable".StartsWith(enable_disable_opt))
                            SkipNet.Parameters.cTableEnabled = enable;
                        else if ("ping".StartsWith(enable_disable_opt))
                            SkipNet.Parameters.pingEnabled = enable;
                        else if ("simpleping".StartsWith(enable_disable_opt))
                            SkipNet.Parameters.smartPingEnabled = ! enable;
                        else
                            break;
                    }
                    else
                        break;
                }
                else
                    break;
            }

            return i;
        }

        // Print the common portion of the usage message
        public static void PrintUsage()
        {
            Console.WriteLine("  General Common Parameters:");
            Console.WriteLine("    [-Seed #] -- Set the seed for random numbers");
            Console.WriteLine("  SkipNet-specific parameters:");
            Console.WriteLine("    [-base #] -- Base of random id (>=2). Default is {0}.", SkipNet.Parameters.b);
            Console.WriteLine("    [-{0}dense] -- Enable or disable the dense r-table.  Default is {1}.", "{e,d}",
                SkipNet.Parameters.denseRTable ? "enabled": "disabled");
            Console.WriteLine("    [-{0}ptable] -- Enable or disable the p-table.  Default is {1}.", "{e,d}",
                SkipNet.Parameters.proximityTableEnabled ? "enabled": "disabled");
            Console.WriteLine("    [-{0}ctable] -- Enable or disable the c-table.  Default is {1}.", "{e,d}",
                SkipNet.Parameters.cTableEnabled ? "enabled": "disabled");
            Console.WriteLine("    [-{0}ping] -- Enable or disable pinging.  Default is {1}.", "{e,d}",
                SkipNet.Parameters.pingEnabled ? "enabled": "disabled");
            Console.WriteLine("    [-{0}simpleping] -- Enable or disable simple pings.  Default is {1}.", "{e,d}",
                (! SkipNet.Parameters.smartPingEnabled) ? "enabled": "disabled");
        }

        // Set the parameter values based on the arguments selected
        public static void SetParameters()
        {
        }

        // Print the arguments selected and information about them
        public static void PrintArgs()
        {
            Console.WriteLine("    -Seed {0}", seed);
            Console.WriteLine("    -base {0}", SkipNet.Parameters.b);
            Console.WriteLine("    -{0}dense (r-table)", SkipNet.Parameters.denseRTable ? "e" : "d");
            Console.WriteLine("    -{0}ptable", SkipNet.Parameters.proximityTableEnabled ? "e" : "d");
            Console.WriteLine("    -{0}ctable", SkipNet.Parameters.cTableEnabled ? "e" : "d");
            Console.WriteLine("    -{0}ping", SkipNet.Parameters.pingEnabled ? "e" : "d");
            Console.WriteLine("    -{0}simpleping", (! SkipNet.Parameters.smartPingEnabled) ? "e" : "d");
            Console.WriteLine("    denseRTable = {0}", SkipNet.Parameters.denseRTable);
            Console.WriteLine("    aggressiveFixups = {0}", SkipNet.Parameters.aggressiveFixups);
            Console.WriteLine("    clbTableEnabled = {0}", SkipNet.Parameters.clbTableEnabled);
            Console.WriteLine("    pTableUpdatePeriod = {0}", SkipNet.Parameters.pTableUpdatePeriod);
            Console.WriteLine("    cTableUpdatePeriod = {0}", SkipNet.Parameters.cTableUpdatePeriod);
            Console.WriteLine("    clbTableUpdatePeriod = {0}", SkipNet.Parameters.clbTableUpdatePeriod);
            Console.WriteLine("    neighbourPingPeriod = {0}", SkipNet.Parameters.neighbourPingPeriod);
        }
    }
}
