//
// Define classes to be provided by both the simulator and cluster environments.
//

using System;
using MessageRouting;
using SkipNet=Microsoft.Research.Herald.SkipNet;

namespace Microsoft.Research.Herald.Common
{
	/// <summary>
	/// Environment-independent properties of the execution environment
	/// </summary>
	public interface ExecutionEnvironment
	{
        ExecutionEnvironmentType Type();        // Returns execution environment type
        bool AmNode(int nodeNumber);            // Test whether we are the specified node number.  Always true on simulator.
        SkipNet.Node GetNode(int nodeNumber);   // Returns the node pointer for the specified node, if we are that node.
        int NodeCount {get; set;}               // Total number of nodes to be used in the experiment
        void Go();                              // Proceed with queued experiment steps
        void GoFor(int milliseconds);           // Run experiment steps for the specified time duration
        NodePhysicalAddress NodePhysAddr(int nodeNumber); // Gets an address we can use to send messages to a node
        SkipNet.NodeIdAddressBind NodeIdAddrBind(int nodeNumber); // Gets a NodeIdAddressBind for an arbitary node
        object OverlayLock {get;}               // Must be locked during calls that execute event handlers
	}

    public enum ExecutionEnvironmentType {SimulatorEnvironment, ClusterEnvironment};
}
