using System;
using System.Collections;
using System.Diagnostics;
using System.Net;
using System.Xml.Serialization;

using MessageRouting;

namespace Microsoft.Research.Taro {

    [Serializable, XmlRootAttribute("TaroAddress",Namespace=TaroConfig.TARO_NAMESPACE)]
    public class TaroAddress : NodePhysicalAddress {
        private string	address;
        private long	ipaddr;
        private int		port;

        public TaroAddress() {}
        public TaroAddress( string address ) {
            SetAddress(address);
        }

        private void SetAddress( string address ) {
            this.address = address;
            if( ! address.StartsWith("soap://") ) {
                // Address is an (IP Address,Port) pair: "192.168.0.1:80"
                int colon = address.IndexOf(':');
                Debug.Assert( colon!=-1 );
                string ipStr = address.Substring(0, colon);
                System.Net.IPAddress ipAddr = System.Net.IPAddress.Parse(ipStr);
                this.ipaddr = ipAddr.Address;
                this.port = Int32.Parse( address.Substring(colon+1) );
            }
        }

        public string GetHostname() {
            if( address.StartsWith("soap://") ) {
                // string is of the form "soap://foo/bar/baz...". extract "foo"
                Char[] Separators = new Char[]{'/'};
                string[] chunks = address.Split( Separators, 4 );
                Debug.Assert( chunks.Length==4 );
                return chunks[2];
            } else {
                // string is of the form "192.168.200.1:1000". extract "192.168.200.1"
                return ipaddr.ToString();
            }
        }

        [XmlElementAttribute("Address",
             Namespace=MessageRouting.SerializationConfig.MESSAGEROUTING_NAMESPACE)]
        public override string Address {
            get { return address; }
            set { SetAddress(value); }
        }

        [XmlElementAttribute("Port",
             Namespace=MessageRouting.SerializationConfig.MESSAGEROUTING_NAMESPACE)]
        public override int Port {
            get { return port; }
            set { port = value; }
        }

        public long BinaryIPAddress {
            get { return this.ipaddr; }
        }
		
        public override string ToString() {
            return address;
        }
	
        public override bool Equals(Object obj) {
            if (!(obj.GetType().Equals(typeof(TaroAddress)))) {
                return false;
            }

            if( String.Compare( ((TaroAddress) obj).address, this.address, true )==0 ) {
                return true;
            } else {
                return false;
            }
        }

        public override int GetHashCode() {
            return this.address.GetHashCode();
        }
    }
}
