using System;

using MessageRouting;

namespace Microsoft.Research.Taro {

    /// <summary>
    /// TaroReceiveMessage is an abstract class for receiving Overlay messages and passing
    /// them up to the LocalEvent handler.
    /// 
    /// It contains an XmlSerializer that is useful for the derived classes to use in
    /// serializing Overlay messages.
    /// </summary>
    public abstract class TaroReceiveMessage {
        // Overlay-related members
        protected TaroLocalEvent		localEvent;
        protected Node					node;

        // Serialization-related members
        protected object serializer;    // Either XmlSerializer or BinaryFormatter

        public TaroReceiveMessage(TaroLocalEvent localEvent, Node node, object serializer) {
            this.localEvent = localEvent;
            this.node = node;
            this.serializer = serializer;
        }

        /// <summary>
        /// This method is invoked when the user exits. We
        /// remove our handlers and so upon returning are no longer accepting commands.
        /// </summary>
        public abstract void ShutDown();
		
    }
}
