using System;
using System.Diagnostics;
using System.IO;
using System.Xml.Serialization;

using MessageRouting;

namespace Microsoft.Research.Taro {

    public delegate void TaroPingCallback(NodePhysicalAddress destination, bool success);

    public abstract class TaroSendMessage : MessageRouting.SendMessage {

        protected object serializer;    // Either XmlSerializer or BinaryFormatter

        // Default constructor
        public TaroSendMessage(object serializer) {
            this.serializer = serializer;
        }

        // Send a message from the source to the destination
        public abstract bool Send(NodeIdAddressBind source, NodeIdAddressBind destination,
            MessageRouting.RouteMsg obj);

        public abstract void BeginSend(NodeIdAddressBind source, NodeIdAddressBind destination,
            MessageRouting.RouteMsg obj, ReplyCallback cb);

        // Send a Ping to the destination
        public abstract bool Ping( NodePhysicalAddress source, NodePhysicalAddress dest );

        // Send a Ping to the destination asynchronously
        public abstract void BeginPing( NodePhysicalAddress source, NodePhysicalAddress dest, TaroPingCallback cb );
    }

    //-----------------------------------------------------------------------
    // Message class definitions
    //-----------------------------------------------------------------------

    /// <summary>
    /// This is a ping message
    /// </summary>
    [Serializable, XmlRootAttribute("TaroPing", Namespace=TaroConfig.TARO_NAMESPACE, IsNullable=true)]
    public class TaroPing {
        public string from;
    }
}
