using System;
using System.Collections;
using System.Diagnostics;
using System.IO;
using System.Net;
using System.Text;
using System.Xml.Serialization;

using MessageRouting;
using SkipNet=Microsoft.Research.Herald.SkipNet;
using Microsoft.Research.Herald.Logger;
using Microsoft.Research.Herald.TLog;
using Microsoft.Research.Herald.Common;
using ControlFlow=Microsoft.Research.Herald.Common.ControlFlow;

namespace RPCTest {
    public class App {
 
        // To be initialized by RPCTest_sim or RPCTest_cluster
        public static ExecutionEnvironment exEnv;
        public static ApplicationEnvironment appEnv;

        // Debugging controls
        public static bool DebugOutput = false;
        public static bool LogDetails = false;
        public static bool forceWarmCache = false;

        // Other variables
        static bool quitImmediately = false;
        public static Random random = null;
        
        // RPC variables
        public static SkipNet.Node myServer = null;
        public static ControlFlow.SimpleControlFlow cf = null;

        // Experiment variables.
        public static int numRPCs = 10;
        public static bool failLastRPCDest = false;
 

        //================================ Code for application program steps ================================

        //
        // Run the application.
        //
        public static void Application() {
            // Ping all nodes to confirm they are ready.
            PingNodesTillTheyReply();
            TLog.InitGlobalLoggingNode(App.myServer, App.exEnv);

            if (forceWarmCache) 
            {
                // Experiment.
                int countAllRPCs = 0, countSelfRPCs = 0, countOtherRPCs = 0;
                long totalAllRPCLatency = 0, totalSelfRPCLatency = 0, totalOtherRPCLatency = 0;
                long totalAllRPCLatencyWarm = 0, totalSelfRPCLatencyWarm = 0, totalOtherRPCLatencyWarm = 0;
                long maxAllRPCLatency = 0, maxSelfRPCLatency = 0, maxOtherRPCLatency = 0;
                long maxAllRPCLatencyWarm = 0, maxSelfRPCLatencyWarm = 0, maxOtherRPCLatencyWarm = 0;
                for (int i = 0; i < numRPCs; i++) 
                {
                    int src = random.Next(exEnv.NodeCount);
                    int dst = random.Next(exEnv.NodeCount);
                    if (failLastRPCDest && i == (numRPCs - 1)) 
                    {
                        cf.FailNode(dst);
                    }
                    RPCWarmMsg msg = new RPCWarmMsg(myServer, cf, dst);
                    RPCWarmReplyMsg rMsg = (RPCWarmReplyMsg) cf.SendRequestMsg(exEnv.NodeIdAddrBind(src), msg);
                    if (rMsg == null) 
                    {
                        Log.Write("test", LogLvl.Hi, "RPC {0} between nodes {1} and {2} never returned a result.", i, src, dst);
                    }
                    else 
                    {
                        // Print out timing results for this RPC.
                        Log.Write("test", LogLvl.Hi, "Timing results for RPC {0} between nodes {1} and {2}: {3}, {4}", i, src, dst,
                            rMsg.rpcLatency, rMsg.rpcWarmLatency);
                        if (src == dst) 
                        {
                            totalSelfRPCLatency += rMsg.rpcLatency;
                            totalSelfRPCLatencyWarm += rMsg.rpcWarmLatency;
                            countSelfRPCs++;
                            if (maxSelfRPCLatency < rMsg.rpcLatency) maxSelfRPCLatency = rMsg.rpcLatency;
                            if (maxSelfRPCLatencyWarm < rMsg.rpcWarmLatency) maxSelfRPCLatencyWarm = rMsg.rpcWarmLatency;
                        } 
                        else 
                        {
                            totalOtherRPCLatency += rMsg.rpcLatency;
                            totalOtherRPCLatencyWarm += rMsg.rpcWarmLatency;
                            countOtherRPCs++;
                            if (maxOtherRPCLatency < rMsg.rpcLatency) maxOtherRPCLatency = rMsg.rpcLatency;
                            if (maxOtherRPCLatencyWarm < rMsg.rpcWarmLatency) maxOtherRPCLatencyWarm = rMsg.rpcWarmLatency;
                        }
                        totalAllRPCLatency += rMsg.rpcLatency;
                        totalAllRPCLatencyWarm += rMsg.rpcWarmLatency;
                        countAllRPCs++;
                        if (maxAllRPCLatency < rMsg.rpcLatency) maxAllRPCLatency = rMsg.rpcLatency;
                        if (maxAllRPCLatencyWarm < rMsg.rpcWarmLatency) maxAllRPCLatencyWarm = rMsg.rpcWarmLatency;
                    }
                }

                //double avRPCLatency = ((double)totalAllRPCLatency) / ((double)countAllRPCs);
                //double avWarmRPCLatency = ((double)totalAllRPCLatencyWarm) / ((double)countAllRPCs);

                Log.Write("test", LogLvl.Hi, "");  Log.Write("test", LogLvl.Hi, "");

                Log.Write("test", LogLvl.Hi, "Self RPCs: count {0}, average latency {1}ms, max latency {2}ms",
                    countSelfRPCs, ((double)totalSelfRPCLatency) / ((double)countSelfRPCs), maxSelfRPCLatency);
                Log.Write("test", LogLvl.Hi, "Other RPCs: count {0}, average latency {1}ms, max latency {2}ms",
                    countOtherRPCs, ((double)totalOtherRPCLatency) / ((double)countOtherRPCs), maxOtherRPCLatency);
                Log.Write("test", LogLvl.Hi, "All RPCs: count {0}, average latency {1}ms, max latency {2}ms",
                    countAllRPCs, ((double)totalAllRPCLatency) / ((double)countAllRPCs), maxAllRPCLatency);

                Log.Write("test", LogLvl.Hi, "");  Log.Write("test", LogLvl.Hi, "");
                
                Log.Write("test", LogLvl.Hi, "Warm Self RPCs: count {0}, average latency {1}ms, max latency {2}ms",
                    countSelfRPCs, ((double)totalSelfRPCLatencyWarm) / ((double)countSelfRPCs), maxSelfRPCLatencyWarm);
                Log.Write("test", LogLvl.Hi, "Warm Other RPCs: count {0}, average latency {1}ms, max latency {2}ms",
                    countOtherRPCs, ((double)totalOtherRPCLatencyWarm) / ((double)countOtherRPCs), maxOtherRPCLatencyWarm);
                Log.Write("test", LogLvl.Hi, "Warm All RPCs: count {0}, average latency {1}ms, max latency {2}ms",
                    countAllRPCs, ((double)totalAllRPCLatencyWarm) / ((double)countAllRPCs), maxAllRPCLatencyWarm);

                Log.Write("test", LogLvl.Hi, "");  Log.Write("test", LogLvl.Hi, "");

            } 
            else 
            {
                // Experiment.
                int countAllRPCs = 0, countSelfRPCs = 0, countOtherRPCs = 0;
                long totalAllRPCLatency = 0, totalSelfRPCLatency = 0, totalOtherRPCLatency = 0;
                long maxAllRPCLatency = 0, maxSelfRPCLatency = 0, maxOtherRPCLatency = 0;
                for (int i = 0; i < numRPCs; i++) 
                {
                    int src = random.Next(exEnv.NodeCount);
                    int dst = random.Next(exEnv.NodeCount);
                    if (failLastRPCDest && i == (numRPCs - 1)) 
                    {
                        cf.FailNode(dst);
                    }
                    RPCMsg msg = new RPCMsg(myServer, cf, dst);
                    RPCReplyMsg rMsg = (RPCReplyMsg) cf.SendRequestMsg(exEnv.NodeIdAddrBind(src), msg);
                    if (rMsg == null) 
                    {
                        Log.Write("test", LogLvl.Hi, "RPC {0} between nodes {1} and {2} never returned a result.", i, src, dst);
                    }
                    else 
                    {
                        // Print out timing results for this RPC.
                        Log.Write("test", LogLvl.Hi, "Timing results for RPC {0} between nodes {1} and {2}: {3}", i, src, dst,
                            rMsg.rpcLatency);
                        if (src == dst) 
                        {
                            totalSelfRPCLatency += rMsg.rpcLatency;
                            countSelfRPCs++;
                            if (maxSelfRPCLatency < rMsg.rpcLatency) maxSelfRPCLatency = rMsg.rpcLatency;
                        } 
                        else 
                        {
                            totalOtherRPCLatency += rMsg.rpcLatency;
                            countOtherRPCs++;
                            if (maxOtherRPCLatency < rMsg.rpcLatency) maxOtherRPCLatency = rMsg.rpcLatency;
                        }
                        totalAllRPCLatency += rMsg.rpcLatency;
                        countAllRPCs++;
                        if (maxAllRPCLatency < rMsg.rpcLatency) maxAllRPCLatency = rMsg.rpcLatency;
                    }
                }
                double avRPCLatency = ((double)totalAllRPCLatency) / ((double)countAllRPCs);
                Log.Write("test", LogLvl.Hi, "");  Log.Write("test", LogLvl.Hi, "");

                Log.Write("test", LogLvl.Hi, "Self RPCs: count {0}, average latency {1}ms, max latency {2}ms",
                    countSelfRPCs, ((double)totalSelfRPCLatency) / ((double)countSelfRPCs), maxSelfRPCLatency);
                Log.Write("test", LogLvl.Hi, "Other RPCs: count {0}, average latency {1}ms, max latency {2}ms",
                    countOtherRPCs, ((double)totalOtherRPCLatency) / ((double)countOtherRPCs), maxOtherRPCLatency);
                Log.Write("test", LogLvl.Hi, "All RPCs: count {0}, average latency {1}ms, max latency {2}ms",
                    countAllRPCs, ((double)totalAllRPCLatency) / ((double)countAllRPCs), maxAllRPCLatency);

                Log.Write("test", LogLvl.Hi, "");  Log.Write("test", LogLvl.Hi, "");
            }
            // Tell all nodes to terminate themselves.
            TerminateNodes();
        }
    
        /// <summary>
        /// Ping all nodes to confirm liveness.
        /// </summary>
        static void PingNodesTillTheyReply() 
        {
            for (int i = 1; i < exEnv.NodeCount; i++) 
            {
                // Ping the node for liveness
                long timeoutMs = 4000; // Start with four seconds
                Console.WriteLine("PingNodesTillTheyReply {0} at {1:T}", i, DateTime.Now);
                while (false == cf.ExecuteBasicNodeOpn(i, ControlFlow.BasicNodeOpn.Ping, timeoutMs)) 
                {
                    Console.WriteLine("Timeout while pinging node {0} at {1:T}.  Trying again ...", i, DateTime.Now);
                    timeoutMs *= 2;
                    if (timeoutMs > 10 * 60 * 1000) timeoutMs = 10 * 60 * 1000; // Cap at 10 minutes
                }
            }
            Console.WriteLine("PingNodesTillTheyReply finished at {0:T}", DateTime.Now);
        }

        /// <summary>
        /// Signal all nodes to terminate themselves (i.e. exit their programs).
        /// </summary>
        static void TerminateNodes() {
            for (int i = 1; i < exEnv.NodeCount; i++) {
                Console.WriteLine("Terminating node {0} at {1:T}", i, DateTime.Now);
                cf.ExecuteBasicNodeOpn(i, ControlFlow.BasicNodeOpn.ExitProgram);
            }
        }

        
        //================================ Handle program args and related state ================================

        /// <summary>
        /// Print the various command-line options to RPCTest.
        /// </summary>
        public static void Usage() {
            Console.WriteLine("RPCTest");

            appEnv.envPrintUsage();
            Console.WriteLine("  Common Parameters:");
            CommonParameters.PrintUsage();

            Console.WriteLine("  RPCTest Parameters:");
            Console.WriteLine("    [-f] -- Fail the destination node of the last RPC instance before doing the RPC");
            Console.WriteLine("    [-D] -- Turns on application detailed logging output");
            Console.WriteLine("    [-D1] -- Turns on application debugging output");
            Console.WriteLine("    [-numRPCs #] -- Sets the number of RPC trials to perform");
            Console.WriteLine("    [-cache] -- Does a 1st RPC to warm up the connection cache, and a 2nd to get the latency");
            Console.WriteLine("    [-quit] -- Causes program to exit without waiting for input");
            ExitProg(0);
        }
       
        /// <summary>
        /// Print the parameters that are being used for this experiment
        /// </summary>
        public static void PrintExperimentParameters() {
            Console.WriteLine("Running RPCTest with parameters");

            appEnv.envPrintArgs();
            Console.WriteLine("  Common Parameters:");
            CommonParameters.PrintArgs();

            Console.WriteLine("  RPCTest Parameters:");
            Console.WriteLine("    Fail destination node of last RPC before doing RPC: {0}", failLastRPCDest);
            Console.WriteLine("    Number of RPC trials: {0}", numRPCs);
            if (LogDetails)
                Console.WriteLine("    Detailed logging output enabled");
            if (DebugOutput)
                Console.WriteLine("    Application debugging output enabled");
            if (forceWarmCache)
                Console.WriteLine("    Forcing a warm cache for RTT measurements");
            if (quitImmediately)
                Console.WriteLine("    Exit without waiting for input");
            Console.WriteLine();
        }

        /// <summary>
        /// Process the command-line options.
        /// </summary>
        public static void GetOptions(string[] args, int starting_at) {
            for (int i = starting_at; i < args.Length; i++) {
                if ( (args[i][0] == '-') || (args[i][0] == '/') ) {
                    switch(args[i][1]) {
                        case 'q':
                            quitImmediately = true;
                            break;
                        case 'D':
                            if (args[i].Equals("-D"))
                                LogDetails = true;
                            else {
                                DebugOutput = true;
                            }
                            break;
                        case 'f':
                            failLastRPCDest = true;
                            break;
                        case 'n':
                            if (args[i].Equals("-numRPCs"))
                                numRPCs = Convert.ToInt32(args[++i]);
                            else {
                                Usage();
                            }
                            break;
                        case 'c':
                            forceWarmCache = true;
                            break;
                        default:
                            Usage();
                            break;
                    }
                }
                else
                    Usage();
            }

            //
            // Fixup & verify parameters
            //

            // Deal with environment-specific parameters.
            appEnv.envParameterFixup();
        }


        //======================== Functions to be called from environemnt-specific code ========================

        public static void SetParameterDefaults() {
            exEnv.NodeCount = 100;
            appEnv.envSetParameterDefaults();

            SkipNet.Parameters.b = 8;
            SkipNet.Parameters.denseRTable = true;
            SkipNet.Parameters.proximityTableEnabled = false;
            SkipNet.Parameters.cTableEnabled = false;
            SkipNet.Parameters.clbTableEnabled = false;
            SkipNet.Parameters.pingEnabled = true;

            //Log.SetCategoryFilterLevel(SkipNet.Node.CATEGORY, LogLvl.Med);
            //Log.SetCategoryFilterLevel(SkipNet.Join.CATEGORY, LogLvl.Disabled);
            //Log.SetCategoryFilterLevel(SkipNet.CLBTable.CATEGORY, LogLvl.Lo);
            Log.SetDefaultFilterLevel(LogLvl.Disabled);
            Log.SetCategoryFilterLevel("test", LogLvl.Hi);
  
            Log.SetCategoryFilterLevel("SLOG", LogLvl.Lo);
            Log.SetCategoryFilterLevel("stat", LogLvl.Lo);
            Log.SetCategoryFilterLevel("TLog", LogLvl.Hi);
        }

        public static void SetRandomSeed(int seed) {
            random = new Random(seed);
        }

        public static void PostProcessParameters() {
            if (CommonParameters.seed == -1) {
                CommonParameters.seed = 19600322;
            }
            if (DebugOutput) {
                Log.SetCategoryFilterLevel("dbg", LogLvl.Hi);
            }
            else {
                Log.SetCategoryFilterLevel("dbg", LogLvl.Disabled);
            }
        }

        /// <summary>
        /// Each overlay has various configuration parameters. We set them here.
        /// </summary>
        public static void SetOverlayParameters(bool doRandom) {
            SkipNet.Parameters.pingEnabled = true;
            SkipNet.Parameters.recordMessagePath = false;
            if (doRandom) SkipNet.Parameters.SetSeed(CommonParameters.seed);
        }

        public static void InitNode(SkipNet.Node node, int nodeIndex, SendMessage s, LocalEvent l) {
            //
            // Create an instance of the application on this node.
            //
            // Need to create an instance of SimpleControlFlow on each node so that
            // exit and failNode will work.
            ControlFlow.SimpleControlFlow myCf = new ControlFlow.SimpleControlFlow(node, exEnv, quitImmediately);
            // Store myCf in the application object so that it won't get GC'ed
            DummyApplication da = new DummyApplication(myCf);
            node.RegisterApp(da, typeof(DummyApplication));

            // If this is the master control node then record some useful 
            // information for App.
            if (nodeIndex == 0) {
                myServer = node;
                cf = myCf;
            }
        }

        public static void UseNode(SkipNet.Node node, int nodeIndex) {
            node.ActivateNode(true);
        }

        public static void PrintStats() {
            appEnv.envPrintStats();
        }

        public static void ExitProg(int exitCode) {
            if (!quitImmediately) {
                Console.WriteLine();
                Console.WriteLine("Hit Enter to exit program.");
                Console.ReadLine();
            }
            Environment.Exit(exitCode);
        }
    }


    /// <summary>
    /// Interface to be supplied by both RPCTest_sim and RPCTest_cluster.
    /// </summary>
    public interface ApplicationEnvironment {
        void envPrintUsage();                   // Print environment-specific usage information
        void envPrintArgs();                    // Print environment-specific argument information
        void envPrintStats();    // Print environment-specific stats
        bool envLogDetails {get; set;}          // Turn on/off logging environment-specific details
        void envSetParameterDefaults();         // Set environment-specific parameter defaults
        void envParameterFixup();               // Reconcile environment-specific parameter settings
        void envReset();                        // Reset environment-specific state, such as counters, etc.
        long envGetTime();                      // Return local time.
        void envExit();                         // Exit process.
    }


    public class DummyApplication: Application {

        // Need to keep a copy of this here so that the garbage collector
        // doesn't get rid of it.
        ControlFlow.SimpleControlFlow cf;

        public DummyApplication(ControlFlow.SimpleControlFlow cf) {
            this.cf = cf;
        }

        public override void NewNeighbour(NodeIdAddressBind id) {}
        public override void NeighbourRemoved(NodeIdAddressBind id, bool failure) {}
    }
    
        
    //================================ Messages and classes for the application ================================

    /// <summary>
    /// Needed for XML serialization of messages.
    /// </summary>
    public class SerializationConfig {
        public const string RPCTEST_NAMESPACE = "http://research.microsoft.com/sn/Herald/RPCTEST";
 
        public static Type [] serializationTypes = new Type[] {
                                                                  typeof(RPCMsg),
                                                                  typeof(RPCReplyMsg),
                                                                  typeof(RPCWarmMsg),
                                                                  typeof(RPCWarmReplyMsg),
                                                                  typeof(RTTMsg),
                                                                  typeof(RTTReplyMsg)
                                                              };
    }

    /// <summary>
    /// Message used to request an RPC.
    /// </summary>
    [Serializable,
    XmlRootAttribute("RPCMsg",Namespace=SerializationConfig.RPCTEST_NAMESPACE,IsNullable=true)]
    public class RPCMsg : ControlFlow.RPCRequestMsg {
        
        public int dstNodeIndex;

        public RPCMsg() {}

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="localNode"></param>
        /// <param name="dstNodeIndex"></param>
        public RPCMsg( MessageRouting.Node localNode, ControlFlow.SimpleControlFlow cf, int dstNodeIndex): base(localNode, cf) {
            this.dstNodeIndex = dstNodeIndex;
        }
 

        /// <summary>
        /// Simply reply to indicate we are alive.
        /// </summary>
        /// <param name="overlayNode"></param>
        public override void ArrivedAtDestination( MessageRouting.Node overlayNode ) {

            Log.Write("dbg", LogLvl.Hi, "RPCMsg {0}", overlayNode.GetNodeIdAddressBind.GetNodeId.ToString());
            Log.logStatistics(overlayNode);
            TLog.Write("TLOG", LogLvl.Hi, "Start of RPC trial");
            TLog.WriteGlobal(overlayNode, "TLOG-GLOBAL", LogLvl.Hi, "Start of RPC trial:");

            RTTReplyCallback cb = new RTTReplyCallback(this);
            RTTMsg msg = new RTTMsg(overlayNode, cb);
            overlayNode.SendMessageDirect(msg, App.exEnv.NodeIdAddrBind(this.dstNodeIndex));
        }
    }

    [Serializable,
    XmlRootAttribute("RPCWarmMsg",Namespace=SerializationConfig.RPCTEST_NAMESPACE,IsNullable=true)]
    public class RPCWarmMsg : ControlFlow.RPCRequestMsg 
    {
        
        public int dstNodeIndex;

        public RPCWarmMsg() {}

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="localNode"></param>
        /// <param name="dstNodeIndex"></param>
        public RPCWarmMsg( MessageRouting.Node localNode, ControlFlow.SimpleControlFlow cf, int dstNodeIndex): base(localNode, cf) 
        {
            this.dstNodeIndex = dstNodeIndex;
        }
 

        /// <summary>
        /// Simply reply to indicate we are alive.
        /// </summary>
        /// <param name="overlayNode"></param>
        public override void ArrivedAtDestination( MessageRouting.Node overlayNode ) 
        {

            Log.Write("dbg", LogLvl.Hi, "RPCMsg {0}", overlayNode.GetNodeIdAddressBind.GetNodeId.ToString());
            Log.logStatistics(overlayNode);
            TLog.Write("TLOG", LogLvl.Hi, "Start of RPC trial");
            TLog.WriteGlobal(overlayNode, "TLOG-GLOBAL", LogLvl.Hi, "Start of RPC trial:");

            RTTWarmReplyCallback cb = new RTTWarmReplyCallback(this, true);
            RTTMsg msg = new RTTMsg(overlayNode, cb);
            overlayNode.SendMessageDirect(msg, App.exEnv.NodeIdAddrBind(this.dstNodeIndex));
        }
    }

    /// <summary>
    /// Reply to an RPCMsg.
    /// </summary>
    [Serializable,
    XmlRootAttribute("RPCReplyMsg",Namespace=SerializationConfig.RPCTEST_NAMESPACE,IsNullable=true)]
    public class RPCReplyMsg : OverlayReplyMsg {

        public NodeIdAddressBind Replier;  // Node sending the reply message.
        public long rpcLatency;

        
        public RPCReplyMsg() {
        }

        public RPCReplyMsg( NodeIdAddressBind replier, long rpcLatency, OverlayRequestMsg requestMsg ): base(requestMsg) {
            this.Replier = replier;
            this.rpcLatency = rpcLatency;
        }
    }

    [Serializable,
    XmlRootAttribute("RPCWarmReplyMsg",Namespace=SerializationConfig.RPCTEST_NAMESPACE,IsNullable=true)]
    public class RPCWarmReplyMsg : OverlayReplyMsg 
    {

        public NodeIdAddressBind Replier;  // Node sending the reply message.
        public long rpcLatency;
        public long rpcWarmLatency;

        
        public RPCWarmReplyMsg() 
        {
        }

        public RPCWarmReplyMsg( NodeIdAddressBind replier, long rpcWarmLatency, long rpcLatency, OverlayRequestMsg requestMsg ): base(requestMsg) 
        {
            this.Replier = replier;
            this.rpcLatency = rpcLatency;
            this.rpcWarmLatency = rpcWarmLatency;
        }
    }

    /// <summary>
    /// Message used to measure RPC round-trip latency.
    /// </summary>
    [Serializable,
    XmlRootAttribute("RTTMsg",Namespace=SerializationConfig.RPCTEST_NAMESPACE,IsNullable=true)]
    public class RTTMsg : OverlayRequestMsg {
        
        public NodeIdAddressBind Requestor;


        public RTTMsg() {}

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="localNode"></param>
        public RTTMsg( MessageRouting.Node localNode, OverlayReplyCallback cb): base(localNode, cb, ControlFlow.Parameters.RPCTimeout, false) {
            this.Requestor = localNode.GetNodeIdAddressBind;
        }
 

        /// <summary>
        /// Simply reply to indicate we are alive.
        /// </summary>
        /// <param name="overlayNode"></param>
        public override void ArrivedAtDestination( MessageRouting.Node overlayNode ) {

            Log.Write("dbg", LogLvl.Hi, "RTTMsg {0}", overlayNode.GetNodeIdAddressBind.GetNodeId.ToString());

            OverlayReplyMsg replyMsg = new RTTReplyMsg(overlayNode.GetNodeIdAddressBind, this);
            overlayNode.SendReply(replyMsg, this.Requestor);
        }
    }

    /// <summary>
    /// Reply to an RTTMsg.
    /// </summary>
    [Serializable,
    XmlRootAttribute("RTTReplyMsg",Namespace=SerializationConfig.RPCTEST_NAMESPACE,IsNullable=true)]
    public class RTTReplyMsg : OverlayReplyMsg {

        public NodeIdAddressBind Replier;  // Node sending the reply message.

        
        public RTTReplyMsg() {
        }

        public RTTReplyMsg( NodeIdAddressBind replier, OverlayRequestMsg requestMsg ): base(requestMsg) {
            this.Replier = replier;
        }
    }

    public class RTTReplyCallback: OverlayReplyCallback {

        RPCMsg rpcMsg;
        long startTime;

        public RTTReplyCallback(RPCMsg rpcMsg) {
            this.rpcMsg = rpcMsg;
            this.startTime = App.appEnv.envGetTime();
        }

        public override void Execute( MessageRouting.Node overlayNode ) {
            Log.Write("dbg", LogLvl.Hi, "RTTReplyCallback");
            Log.logStatistics(overlayNode);
            long rpcLatency = 0;
            RTTReplyMsg reply = (RTTReplyMsg) this.Reply;
            if (reply == null) rpcLatency = -1;
            else {
                long now = App.appEnv.envGetTime();
                rpcLatency = now - this.startTime;
            }
            TLog.Write("TLOG", LogLvl.Hi, "End of RPC trial.  RTT: {0}", rpcLatency);
            TLog.WriteGlobal(overlayNode, "TLOG-GLOBAL", LogLvl.Hi, "End of RPC trial");

            OverlayReplyMsg replyMsg = new RPCReplyMsg(overlayNode.GetNodeIdAddressBind, rpcLatency, this.rpcMsg);
            overlayNode.SendReply(replyMsg, this.rpcMsg.Requestor);
        }
    }
    public class RTTWarmReplyCallback: OverlayReplyCallback 
    {

        RPCWarmMsg rpcMsg;
        long startTime;
        bool isInitialRTT;
        long rpcColdLatency;

        public RTTWarmReplyCallback(RPCWarmMsg rpcMsg, bool isInitialRTT) 
        {
            this.rpcMsg = rpcMsg;
            this.startTime = App.appEnv.envGetTime();
            this.isInitialRTT = isInitialRTT;
        }

        public override void Execute( MessageRouting.Node overlayNode ) 
        {
            Log.Write("dbg", LogLvl.Hi, "RTTReplyCallback");
            Log.logStatistics(overlayNode);
            long rpcLatency = 0;
            RTTReplyMsg reply = (RTTReplyMsg) this.Reply;
            if (reply == null) rpcLatency = -1;
            else 
            {
                long now = App.appEnv.envGetTime();
                rpcLatency = now - this.startTime;
            }
            if (isInitialRTT) 
            {
                RTTWarmReplyCallback cb = new RTTWarmReplyCallback(rpcMsg, false);
                cb.rpcColdLatency = rpcLatency;
                RTTMsg msg = new RTTMsg(overlayNode, cb);
                overlayNode.SendMessageDirect(msg, App.exEnv.NodeIdAddrBind(rpcMsg.dstNodeIndex));
            } 
            else 
            {
                TLog.Write("TLOG", LogLvl.Hi, "End of RPC trial.  RTT: {0}", rpcLatency);
                TLog.WriteGlobal(overlayNode, "TLOG-GLOBAL", LogLvl.Hi, "End of RPC trial");

                OverlayReplyMsg replyMsg = new RPCWarmReplyMsg(overlayNode.GetNodeIdAddressBind, rpcLatency, rpcColdLatency, this.rpcMsg);
                overlayNode.SendReply(replyMsg, this.rpcMsg.Requestor);
            }
        }
    }
}
