//
// Implementent classes to be provided by both the simulator and cluster environments.
//

using System;
using MessageRouting;
using SkipNet=Microsoft.Research.Herald.SkipNet;
using Microsoft.Research.Herald.Common;

using Simulator;
using Microsoft.Research.Herald.SimLib;

namespace Microsoft.Research.Herald.SimLib
{
	/// <summary>
	/// Simulator implementation of environment-independent properties of the execution environment
	/// </summary>
    public class SimExecutionEnvironment : ExecutionEnvironment
    {
        static SimInterface s
        {
            get { return SimNodes.s; } // Allow use of just "s" instead of "SimNodes.s"
        }

        /// <summary>
        /// Returns execution environment type
        /// </summary>
        public ExecutionEnvironmentType Type()
        {
            return ExecutionEnvironmentType.SimulatorEnvironment;
        }

        /// <summary>
        /// Test whether we are the specified node number.  Always true on simulator.
        /// </summary>
        public bool AmNode(int nodeNumber)
        {
            return nodeNumber < SimLibParameters.nodes;
        }

        /// <summary>
        /// Returns the node pointer for the specified node, if we are that node.
        /// </summary>
        public SkipNet.Node GetNode(int nodeNumber)
        {
            return (SkipNet.Node) s.GetNodeArray[nodeNumber];
        }

        /// <summary>
        /// Total number of nodes to be used in the experiment.
        /// </summary>
        public int NodeCount
        {
            get {return SimLibParameters.nodes;}
            set {SimLibParameters.nodes = value;}
        }

        /// <summary>
        /// Proceed with queued experiment steps
        /// </summary>
        public void Go()
        {
            s.ProcessUntilIgnoreEventsOnly();
        }
        
        /// <summary>
        /// Run experiment steps for the specified time duration
        /// </summary>
        public void GoFor(int milliseconds)
        {
            s.ProcessForTime(milliseconds);
        }

        // Gets an address we can use to send messages to a node
        public NodePhysicalAddress NodePhysAddr(int nodeNumber)
        {
            return GetNode(nodeNumber).GetPhysicalAddr;
        }

        // Gets a NodeIdAddressBind for an arbitary node
        public SkipNet.NodeIdAddressBind NodeIdAddrBind(int nodeNumber)
        {
            return (SkipNet.NodeIdAddressBind) GetNode(nodeNumber).GetNodeIdAddressBind;
        }

        static object something_to_lock_even_though_we_are_single_threaded = new int [0];

        // Must be locked during calls that execute event handlers
        public object OverlayLock 
        {
            get { return something_to_lock_even_though_we_are_single_threaded; }
        }    
    }
}
