//
// Parse, store, and print parameters specific to the simulator environment.
//

using System;
using System.Collections;
using System.Diagnostics;
using System.IO;

namespace Microsoft.Research.Herald.SimLib
{
    public class SimLibParameters
    {
        public static int nodes = 200;
        public static int numOrgs = 10;
        public static bool clusterByOrg = false;
        public static bool clusterOrgsByZipf = false;
        public static bool dataClusterByOrg = false;
        public static bool dataClusterOrgsByZipf = false;
        public static bool useRealMachineNamesForNodes = false;
        public static bool useRealOrgNames = false;
        public static bool extended_simulator_model = false;
        public static string topology = "merc";    // Formerly "600"

        // Parse simulator-specific command line arguments from the specified index,
        // returning the index of the first unparsed parameter.
        // Returns -1 if Usage() needs to be called.
        public static int ParseArgs(string [] args, int starting_at)
        {
            int i = starting_at;
            for (; i < args.Length; i++) 
            {
                if ( (args[i][0] == '-') || (args[i][0] == '/') ) 
                {
                    string opt = args[i].Substring(1);
                    if (opt.Length < 1) break;

                    if (opt.StartsWith("names")) 
                    {
                        if (args[i+1].IndexOf("h") >= 0)
                            useRealMachineNamesForNodes = true;
                        if (args[i+1].IndexOf("o") >= 0)
                            useRealOrgNames = true;
                        i++;
                    }
                    else if ("nodes".StartsWith(opt))
                    {
                        nodes = Convert.ToInt32(args[++i]);
                    }
                    else if (opt.StartsWith("dCluster")) 
                    {
                        dataClusterByOrg = true;
                        if (opt.StartsWith("dClusterZ")) 
                        {
                            dataClusterOrgsByZipf = true;
                        }
                    } 
                    else if ("extended".StartsWith(opt))
                    {
                        extended_simulator_model = true;
                    }
                    else if ("topology".StartsWith(opt))
                    {
                        topology = args[++i];
                    }
                    else if ("orgs".StartsWith(opt)) 
                    {
                        numOrgs = Convert.ToInt32(args[++i]);
                    }
                    else if (opt.StartsWith("cluster")) 
                    {
                        clusterByOrg = true;
                        if (opt.StartsWith("clusterZ")) 
                        {
                            clusterOrgsByZipf = true;
                        }
                    }
                    else
                        break;
                }
                else
                    break;
            }

            return i;
        }
        
        // Print the common portion of the usage message
        public static void PrintUsage()
        {
            Console.WriteLine("  Simulator-specific Parameters:");
            Console.WriteLine("    [-nodes #] -- Default {0}", nodes);
            Console.WriteLine("    [-orgs[N] #] -- Default {0}", numOrgs);
            Console.WriteLine("    [-names [h][o] ~ h => use real host names, o => use real org names");
            Console.WriteLine("    [-topology {0}] -- Default {1}", "{600, 5050, merc}", topology);
            Console.WriteLine("    [-extended-simulation-modeling");
            Console.WriteLine("    [-cluster[Z[N]]]");
            Console.WriteLine("    [-dCluster[Z]]");
        }

        // Set the parameter values based on the arguments selected
        public static void SetParameters()
        {
        }

        // Print the arguments selected and information about them
        public static void PrintArgs()
        {
            Console.WriteLine("  Simulator-specific parameters:");
            Console.WriteLine("    -nodes {0}", nodes);
            Console.WriteLine("    -orgs {0}", numOrgs);
            Console.WriteLine("    -topology {0}", topology);
            Console.WriteLine("    -extended-simulation-modeling {0}", SimLibParameters.extended_simulator_model);

            Console.WriteLine("    -clusterByOrg {0}", clusterByOrg);
            Console.WriteLine("    -clusterOrgsByZipf {0}", clusterOrgsByZipf);
            Console.WriteLine("    -dataClusterByOrg {0}", dataClusterByOrg);
            Console.WriteLine("    -dataClusterOrgsByZipf {0}", dataClusterOrgsByZipf);
            Console.WriteLine("    -useRealMachineNamesForNodes {0}", useRealMachineNamesForNodes);
            Console.WriteLine("    -useRealOrgNames {0}", useRealOrgNames);
        }
    }
}