using System;
using MessageRouting;
using System.Collections;
using System.Diagnostics;

namespace Simulator
{
    /// <summary>
    /// Summary description for Simulator.
    /// </summary>
    public class Sim2 : SendMessage, LocalEvent, TransportLayer
    {
        private EventQueue eventq;
        private SimTopology top;
        private Node[] nodes;
        private int nodePtr = 0;
        private long eventsIn = 0;
        private long localEvents = 0;
        private CheckIgnore local;
        //public bool D_FLAG = false;

        public static int[] nodeNumMsgsReceived;

        public Sim2(int n, CheckIgnore ignore)
        {
            eventq = new EventQueue();
            top = new SimTopology(n);
            nodes = new Node[n];
            this.local = ignore;

            if (SimParameters.TrackMessageReceipts) 
            {
                nodeNumMsgsReceived = new int[n];
                ResetStats();
            }
        }

        public void ResetSimulator()
        {
            //          eventq = new EventQueue();
            //          timeNow = 0L;
        }

        // Return an array of numbers of messages received by each node.
        // The index into the array is the same as the one to use with GetNodeArray.
        public int[] GetNodeMessageReceiveStats()
        {
            return nodeNumMsgsReceived;
        }
        public long GetNumMessagesSentToWire(Node node) 
        {
            Debug.Assert(false);
            return 0;
        }
        public long GetNumMessagesReceivedFromWire(Node node)
        {
            Debug.Assert(false);
            return 0;
        }


        public void ResetStats() 
        {
            if (nodeNumMsgsReceived == null) return;

            for (int i = 0; i < nodeNumMsgsReceived.Length; i++) 
            {
                nodeNumMsgsReceived[i] = 0;
            }
        }

        public Node[] GetNodeArray 
        { 
            get 
            { 
                if (SimParameters.SupportMultipleOverlayNetworks) throw new Exception("Can't use GetNodeArray if multiple overlay networks are allowed");
                return nodes; 
            } 
        }

        public Node[] GetOverlayNodeArray(OverlayId oid)
        {
            Node[] singleNetworkNodes = new Node[nodes.Length];
            int j = 0;
            for (int i = 0; i < nodePtr; i++)
            {
                if (nodes[i].OverlayId.idGlobal == oid.idGlobal &&
                    nodes[i].OverlayId.idLocal == oid.idLocal) 
                {
                    singleNetworkNodes[j++] = nodes[i];
                }
            }
            return singleNetworkNodes;
        }
        
        public SimTopology GetNetTopology { get { return top; } 
        }

        public Topology GetTopology()
        {
            return top;
        }
        
        public long GetTime() { return eventq.GetTime; } 
        
        long diststat = 0;

        public long DistanceStat
        {
            get { return diststat; }
            set { diststat = 0; }
        }

        public virtual bool Send(NodeIdAddressBind source, NodeIdAddressBind dest, MessageRouting.RouteMsg obj)
        {
            if (source.GetAddress.Failed || dest.GetAddress.Failed)
                return false;
            // Note: during initialization dest.GetNodeId may be null, so we check for that special-case.
            int d = top.Delay(source.GetAddress,dest.GetAddress);
            if (d < 0)
                return false; // The node no longer exists
            diststat += d;
            eventsIn++;
            if (local.IgnoreThis(obj))
                localEvents++;
            eventq.InsertObject(eventq.GetTime+d,new Event(dest,obj));
            //if (Pastry.Parameters.DEBUG_PASTRY)
            //    Console.WriteLine("LOW_LEVEL SEND: " + ((Pastry.NodeIdAddressBind) source).NodeId +" -> "+ ((Pastry.NodeIdAddressBind) dest).NodeId +" " + ((RouteMsg) obj).MessageType + " " + d + " "+ eventq.GetTime + " " + (eventq.GetTime+d));
            return true;
        }

        // Send a message from the source node to the destination node asynchronously
        public void BeginSend(MessageRouting.NodeIdAddressBind source, 
            MessageRouting.NodeIdAddressBind destination, MessageRouting.RouteMsg obj, ReplyCallback cb) {
            bool success = Send(source, destination, obj);
            cb(destination, success, (MessageRouting.RouteMsg) obj);
        }

        public void RaiseEvent(long ms, NodeIdAddressBind me, MessageRouting.RouteMsg obj)
        {
            eventsIn++;
            if (local.IgnoreThis(obj))
                localEvents++;
            eventq.InsertObject(eventq.GetTime+ms,new Event(me,obj));
            //if (E_FLAG)
            //  Console.WriteLine("LOW_LEVEL SEND: " + source.NodeId +" -> "+ dest.NodeId +" " + ((RouteMsg) obj).MessageType);
        }

        public void RemoveEvent(MessageRouting.RouteMsg message) {
            Event dummy = new Event(null, message);
            eventq.Remove(dummy);
        }

        public Node CreateNode(Node node, NodePhysicalAddress addr, Object nodeInitData)
        {
            addr.AttachLogicalNode(node);
            nodes[nodePtr] = node;
            nodes[nodePtr].InitNode(this, this, top, addr, (NodePhysicalAddress) nodeInitData);
            return nodes[nodePtr++];
        }
/*
        public Node CreateNode()
        {
            if (nodePtr == 0)
            {
                nodes[nodePtr] = new Pastry.Node();
                SimNodePhysicalAddressOld addr = new SimNodePhysicalAddressOld();           
                addr.AttachLogicalNode(nodes[nodePtr]);
                nodes[nodePtr].InitNode(this, this, top, addr, null);  // first node
                return nodes[0];
            }
            else
            {
                nodes[nodePtr] = new Pastry.Node();
                SimNodePhysicalAddressOld addr = new SimNodePhysicalAddressOld();           
                addr.AttachLogicalNode(nodes[nodePtr]);
                NodePhysicalAddress seedNeighbors;
                if (Pastry.Parameters.UseNetworkLocality)
                    seedNeighbors = top.GetClosest(addr);
                else
                    seedNeighbors = top.GetRandom();
                nodes[nodePtr].InitNode(this, this, top, addr, seedNeighbors);
                return nodes[nodePtr++];
            }
            //          nodes[nodePtr] = node;
            //          nodes[nodePtr].InitNode(this, this, top, addr, nodeInitData);
            //          return nodes[nodePtr++];
        }
*/
        public void ProcessAllEvents() {
            for(;;) {
                Event n = (Event) eventq.GetNextEvent();
                if( n==null ) break;
                ProcessEvent(n);
            }
        }

        public void ProcessUntilIgnoreEventsOnly() {
            for(;;) {
                if(eventsIn - localEvents <= 0) break;
                Event n = (Event) eventq.GetNextEvent();
                if( n==null ) break;
                ProcessEvent(n);
            }
        }

        public void ProcessForTime(long ms) {
            long endTime = eventq.GetTime + ms;
            //  Console.WriteLine("NOW: " + eventq.GetTime + " END: " + endTime);
            for(;;) {
                Event n = (Event) eventq.GetNextEvent();
                if( n==null ) break;
                if( eventq.GetTime>endTime ) {
                    // Oops: This event fires in the future
                    eventq.InsertObject(eventq.GetTime, n);
                    break;
                }
                ProcessEvent(n);
            }
        }

        private void ProcessEvent(Event n)
        {
            eventsIn--;
            if (local.IgnoreThis(n.GetMessage))
                localEvents--;
            else if (SimParameters.TrackMessageReceipts)
                nodeNumMsgsReceived[((SimNodePhysicalAddressOld)(n.GetDestination.GetAddress)).Port] += 1;

            if (!n.GetDestination.GetAddress.Failed) 
            {
                //              Console.WriteLine("OUT: " + timeNow);
                //if (Pastry.Parameters.DEBUG_PASTRY)
                //    Console.WriteLine("LOW_LEVEL RAISE: " + n.GetDestination.GetNodeId + " " + ((RouteMsg) n.GetMessage).MessageType + " " + eventq.GetTime);
                RouteMsg msg = (RouteMsg) n.GetMessage;
                if (SimParameters.SupportMultipleOverlayNetworks)
                {
                    OverlayId msgOid = msg.Overlay;
                    n.GetDestination.GetAddress.GetLogicalNode(msgOid).RaiseEventUpcall(msg);
                }
                else
                    ((SimNodePhysicalAddressOld)(n.GetDestination.GetAddress)).Node.RaiseEventUpcall(msg);
            }
        }

    }
}
