using System;
using MessageRouting;
using System.Collections;

namespace Simulator {

    /// <summary>
    ///    Summary description for SimNodePhysicalAddress.
    /// </summary>
    [Serializable]
    public class SimNodePhysicalAddress : NodePhysicalAddress {
        [NonSerialized]
        private ArrayList nodes;
        private int nodeNum;
	
        public SimNodePhysicalAddress(int i) {
            nodes = new ArrayList(1);
            nodeNum = i;
        }

        public override void AttachLogicalNode(Node n) {
            nodes.Add(n);
        }

        public override Node GetLogicalNode(OverlayId oid) {
            for (int i = 0; i < nodes.Count; i++) {
                Node n = (Node) nodes[i];
                if (n.OverlayId.idGlobal == oid.idGlobal &&
                    n.OverlayId.idLocal == oid.idLocal) {
                    return n;
                }
            }
            return null;
        }

        public override string Address {
            get {return nodeNum.ToString();}  // We use nodeNum as a bogus host address.
            set {throw new Exception("Not implemented");}
        }

        public override int Port {
            get {return nodeNum;}  // Use nodeNum as surrogate port value.
        }
		
        public int GetNum {
            get {
                return nodeNum;
            }
        }
		
        public Node Node {
            get {
                if (SimParameters.SupportMultipleOverlayNetworks) throw new Exception("Can't use Node property if multiple overlay networks are allowed: use GetNode(oid) instead");
                return (Node)nodes[0];
            }
        }

			
        public override string ToString() {
            return ""+nodeNum;
        }
	
        public override bool Equals(Object obj) {
            if (!(obj.GetType().Equals(typeof(SimNodePhysicalAddress))))
                return false;
            return ( ((SimNodePhysicalAddress) obj).nodeNum == this.nodeNum);
        }	

        public override int GetHashCode() {
            return nodeNum;
        }
    }
}


