using System;
using MessageRouting;
using System.Collections;

namespace Simulator
{
	/// <summary>
	/// Summary description for SimPhysicalAddressOld.
	/// </summary>
	public class SimNodePhysicalAddressOld : NodePhysicalAddress 
	{
		private static Node[] nodes = new Node[100000];
		private static int totalNodes = 0;
	
		private Node node;
		private int nodeNum;
	
		public SimNodePhysicalAddressOld() 
		{
			this.nodeNum = totalNodes++;
		}

	    public override string Address {
		get {return node.ToString();}  // We use a bogus host address.
		set {throw new Exception("Not implemented");}
	    }

	    public override int Port {
		get {return nodeNum;}  // Use nodeNum as surrogate port value.
	    }
	
		static void Reset()
		{
			nodes = new Node[100000];
			totalNodes = 0;
		}
	
		// return total number of existing nodes
		public static int GetTotalNodes { get { return totalNodes;} 
		}
	
		// get the address of Node with number num
		public static SimNodePhysicalAddressOld GetNode(int num) 
		{
			Node n = (Node)nodes[num];	
			if (n != null) 
			{
				return (SimNodePhysicalAddressOld)n.GetPhysicalAddr;
			}
			return null;
		}
		
		public int GetNum { get { return nodeNum; }
		}

		public override string ToString()
		{
			return ""+nodeNum;
		}

		public override void AttachLogicalNode(Node n) 
		{
			node = n;
			nodes[nodeNum] = n;
		}

	    public override Node GetLogicalNode(OverlayId oid)
	    {
		throw new Exception("GetLogicalNode has not been implemented for SimNodePhysicalAddressOld");
	    }
	
		// NodePhysicalAddress methods
	
		// obtain the node object handle associated with this physical address
		public Node GetNode() {return node;}
			
		public Node Node { 
		    get { 
			if (SimParameters.SupportMultipleOverlayNetworks) throw new Exception("Can't use Node property if multiple overlay networks are allowed: use GetNode(oid) instead");
			return node; 
		    } 
		}

		public override bool Equals(Object n)
		{
			if (!(n is SimNodePhysicalAddressOld)) return false;
			if ( ((SimNodePhysicalAddressOld) n).nodeNum == this.nodeNum)
				return true;
			return false;
		}
	}
}
