using System;
using System.Security.Cryptography;

namespace Simulator
{
    public class SimParameters
    {
        // True => simulator can support multiple overlay networks.
        public static bool SupportMultipleOverlayNetworks = false;  

        // True => simulator will model link bandwidth effects.
        public static bool ModelLinkBandwidth = false;
        public const int DefaultTransitTransitBandwidth = 250000;  // 2 Gbps = 2000 * 1000 / 8 Kbytes/sec
        public const int DefaultTransitStubBandwidth = 5625;  // 45 Mbps = 45 * 1000 / 8 Kbytes/sec

        public const int OC3Bandwidth = 19375;  // 155 Mbps = 155 * 1000 / 8 Kbytes/sec
        public const int MinOC3Latency = 10;  // msecs
        public const int MaxOC3Latency = 40;  // msecs

        public const int T3Bandwidth = 5625;  // 45 Mbps = 45 * 1000 / 8 Kbytes/sec
        public const int MinT3Latency = 300; // msecs
        public const int MaxT3Latency = 500; // msecs

        public const double FractionOfOC3 = 0.97; // remainder are T3

        public const int DefaultStubStubBandwidth = 12500;  // 100 Mbps = 100 * 1000 / 8 Kbytes/sec
        public const int MinStubStubLatency = 1; // msecs
        public const int MaxStubStubLatency = 2; // msecs

        public const int DefaultLanBandwidth = 12500;  // 100 Mbps = 100 * 1000 / 8 Kbytes/sec
        public const int DefaultLanLatency = 1;  // msecs

        // True => simulator enqueues messages at each en-route router node
        // in order to more accurately model link congestion.
        public static bool ModelIntermediateLinkQueuing = false;

        // True => simulator keeps track of how many messages each node receives.
        public static bool TrackMessageReceipts = true;

        // True => probabilistically drop messages according to the percentage specified.
        public static bool DropMessages = false;
        public static double DropFraction = 0.0; // [0.0 - 1.0) DropMessages == true => drop this fraction of messages

        private static ulong rand = 45743UL; // This is used where ever random numbers are needed...
        private static SHA1CryptoServiceProvider md = new SHA1CryptoServiceProvider();

        public static int Random 
        { 
            get 
            {
                ulong n = rand++;
                byte[] data = new Byte[8];
                for (int i = 0; i < data.Length; i++)
                {
                    data[i] = (byte) (n & 0xFF);
                    n >>= 8;
                }
                byte[] digest = md.ComputeHash(data);
                int result = 0;
                for (int i = 0; i < 4; i++)
                    result = (result << 8) + digest[i];
                // Maths.Abs can not handle a value of Int32.MinValue - so to save stress we remove them
                if (result == Int32.MinValue)
                    result++;

                return result;
            }
            //get { return rand.Next(); } 
            set { rand = (ulong) value; }
        }
    }
}
