//#define DEMO

namespace Simulator
{
    using System;
    using MessageRouting;
    using System.Collections;

    /// <summary>
    ///    Summary description for SimTopology.
    /// </summary>
    public class SimTopology : Topology
    {
#if DEMO
        const bool sphere = false; 
        const int MaxCoord = 100;   
#else 
        const bool sphere = true; // FOR DEMO = false
        const int MaxCoord = 1000; // For DEMO = 100;   
#endif
        const double radius = 1000.0;
        const bool threeD = false;
    
        public static int MaxDist = (sphere ? (int) (radius*2*Math.PI) : (int) Math.Sqrt(MaxCoord * MaxCoord * (threeD ? 3 : 2) ) );

        public static bool record = false;
        public static bool heavyrecord = false;
        public static ArrayList src = new ArrayList();
        public static ArrayList dest = new ArrayList();
        public static Hashtable all = null;
        
        private short[] xCoordinate;
        private short[] yCoordinate;
        private short[] zCoordinate;
        private double[] longtitude;
        private double[] latitude;
        private int numNodes;
    
        public SimTopology(int numNodes) 
        {
            if (sphere)
            {
                longtitude = new double[numNodes];
                latitude = new double[numNodes];
            }
            else
            {
                xCoordinate = new short[numNodes];
                yCoordinate = new short[numNodes];
                zCoordinate = new short[numNodes];
            }
            this.numNodes = numNodes;
        
            // create a random location matrix
            Random r = new Random(SimParameters.Random);

            for (int i=0; i<numNodes; i++ ) 
            {
                if (sphere)
                {
                    double z = (r.NextDouble() * (2*radius)) - radius;
                    longtitude[i] = Math.Abs(r.NextDouble()) * (2.0*Math.PI);
                    latitude[i] = Math.Asin(z/radius);
                }
                else
                {
                    xCoordinate[i] = (short) Math.Abs(r.Next()%MaxCoord);
                    yCoordinate[i] = (short) Math.Abs(r.Next()%MaxCoord);

                    if (threeD)
                        zCoordinate[i] = (short) Math.Abs(r.Next()%MaxCoord);
                    else
                        zCoordinate[i] = 0;
                }
                // Check if there is a node at the same coordinates
                for (int j=0; j < i; j++) 
                {
                    if (sphere)
                    {
                        if (latitude[i] == latitude[j] && longtitude[i] == longtitude[j])
                        {
                            Console.WriteLine("Collision on sphere");
                            i--;
                            break;;
                        }
                    }
                    else
                    {
                        if (xCoordinate[i] == xCoordinate[j] && yCoordinate[i] == yCoordinate[j]) 
                        {
                            i--;
                            break;;
                        }
                    }
                }

            }
            
            //test();
        }
    
        // measure the distance between a and b, relative to the topology of the network and an
        // appropriate distance measure
        public int Ping(NodePhysicalAddress a, NodePhysicalAddress b, bool disableCache) 
        {
            if (a.Failed || b.Failed)
                return -1;
            /*  if (!((SimNodePhysicalAddressOld)a).Node.IsActive)
            {
                Console.WriteLine("DIST: " + ((SimNodePhysicalAddressOld)a).Node.GetNodeIdAddressBind.GetNodeId + " -> " + ((SimNodePhysicalAddressOld)b).Node.GetNodeIdAddressBind.GetNodeId);
                Console.WriteLine("Ping while not active");
            }
        */  if (heavyrecord)
            {
                /*ulong h = (ulong) ((SimNodePhysicalAddressOld)a).GetNum << 32;
                h += (ulong) ((SimNodePhysicalAddressOld)b).GetNum;
                if (!all.ContainsKey(h))
                {
                    all.Add(h,true);
                    if (record)
                    {
                        src.Add(((SimNodePhysicalAddressOld)a).GetNum);
                        dest.Add(((SimNodePhysicalAddressOld)b).GetNum);
                    }
                }*/
                Hashtable p = (Hashtable) all[((SimNodePhysicalAddressOld)a).GetNum];
                if (p == null)
                {
                    p = new Hashtable();
                    all.Add(((SimNodePhysicalAddressOld)a).GetNum,p);
                }
                if (!p.Contains(((SimNodePhysicalAddressOld)b).GetNum))
                {
                    p.Add(((SimNodePhysicalAddressOld)b).GetNum,true);
                    if (record)
                    {
                        src.Add(((SimNodePhysicalAddressOld)a).GetNum);
                        dest.Add(((SimNodePhysicalAddressOld)b).GetNum);
                    }
                }
            }
            else if (record)
            {
                src.Add(((SimNodePhysicalAddressOld)a).GetNum);
                dest.Add(((SimNodePhysicalAddressOld)b).GetNum);
            }
            return Distance(((SimNodePhysicalAddressOld)a).GetNum, ((SimNodePhysicalAddressOld)b).GetNum);
        }

        public void BeginPing(NodePhysicalAddress a, NodePhysicalAddress b, bool disableCache, PingCallback cb) {
            int distance;
            distance = Ping(a, b, disableCache);
            cb(b, distance);
        }
        
        public int Delay(NodePhysicalAddress a, NodePhysicalAddress b)
        {
            return Ping(a,b,true);
        }
    
        public short GetX(NodePhysicalAddress a)
        {
            if (sphere)
                return (short) longtitude[((SimNodePhysicalAddressOld)a).GetNum];
            else
                return xCoordinate[((SimNodePhysicalAddressOld)a).GetNum];
        }
        public short GetY(NodePhysicalAddress a)
        {
            if (sphere)
                return (short) latitude[((SimNodePhysicalAddressOld)a).GetNum];
            else
                return yCoordinate[((SimNodePhysicalAddressOld)a).GetNum];
        }
        public short GetZ(NodePhysicalAddress a)
        {
            return zCoordinate[((SimNodePhysicalAddressOld)a).GetNum];
        }

        public short GetX(int i) {
            return xCoordinate[i];
        }

        public short GetY(int i) {
            return yCoordinate[i];
        }

        // measure the distance between a and b, relative to the topology of the network and an
        // appropriate distance measure
        
        public int Distance(int a, int b) 
        {
            if (sphere)
            {
                return (int) ( radius*Math.Acos(
                                        Math.Cos(longtitude[a]-longtitude[b])*Math.Cos(latitude[a])*Math.Cos(latitude[b]) +
                                        Math.Sin(latitude[a])*Math.Sin(latitude[b])
                    ));
            }
            else
            {
                int xDiff = Math.Abs(xCoordinate[a] - xCoordinate[b]);
                int yDiff = Math.Abs(yCoordinate[a] - yCoordinate[b]);
                int zDiff = Math.Abs(zCoordinate[a] - zCoordinate[b]);
        
                int n =(int) Math.Sqrt( (xDiff * xDiff) + (yDiff * yDiff) + (zDiff * zDiff) );
                return n;
            }
        }
    
        public NodePhysicalAddress GetRandom() 
        {
            if (SimNodePhysicalAddressOld.GetTotalNodes == 0) 
                return null;
        
            int i = Math.Abs(SimParameters.Random) % SimNodePhysicalAddressOld.GetTotalNodes;
            while (true) 
            {
                if ((SimNodePhysicalAddressOld.GetNode(i) != null) && 
                    (SimNodePhysicalAddressOld.GetNode(i).Node.IsActive))
                        return SimNodePhysicalAddressOld.GetNode(i);
                i = (++i) % SimNodePhysicalAddressOld.GetTotalNodes;
            }
        }

        // find the node closest to the given node
        public NodePhysicalAddress GetClosest(NodePhysicalAddress n) 
        {
            int minN = 0;
            int minD = MaxDist;
        
            if (SimNodePhysicalAddressOld.GetTotalNodes == 0) 
                return null;
        
            for (int i=0; i<SimNodePhysicalAddressOld.GetTotalNodes; i++ ) 
            {
                if (SimNodePhysicalAddressOld.GetNode(i) != null)
                {
                    if (SimNodePhysicalAddressOld.GetNode(i).Node.IsActive)
                    {
            //          SimNodePhysicalAddressOld compare = SimNodePhysicalAddressOld.GetNode(i);
            //          x1 = Math.Abs(xCoordinate[compare.GetNum] - xCoordinate[((SimNodePhysicalAddressOld)n).GetNum]);
            //          y1 = Math.Abs(yCoordinate[compare.GetNum] - yCoordinate[((SimNodePhysicalAddressOld)n).GetNum]); 
            //          if ((x1 < x) && (y1 < y))
                        {
                            int tmp = Ping(SimNodePhysicalAddressOld.GetNode(i), (SimNodePhysicalAddressOld)n, true);
                            if (tmp < minD) 
                            {
                                minD = tmp;
                                minN = i;
            //                  x = x1;
            //                  y = y1;
                            }
                        }
                    }
                }
            }
            return SimNodePhysicalAddressOld.GetNode(minN);
        }
    
        public int Distance(SimNodePhysicalAddressOld node, int x, int y, int z) 
        {
            int a = node.GetNum;
            int xDiff = Math.Abs(xCoordinate[a] - x);
            int yDiff = Math.Abs(yCoordinate[a] - y);
            int zDiff = Math.Abs(zCoordinate[a] - z);
                             
            return (int) Math.Sqrt( (xDiff * xDiff) + (yDiff * yDiff) + (zDiff * zDiff) );
        }
    
        // find the node closest to the given node
        public NodePhysicalAddress GetClosest(int x, int y, int z) 
        {
            int minN = 0;
            int minD = MaxDist;
        
            if (SimNodePhysicalAddressOld.GetTotalNodes == 0) return null;
        
            for (int i=0; i<SimNodePhysicalAddressOld.GetTotalNodes; i++ ) 
            {
                int tmp = Distance(SimNodePhysicalAddressOld.GetNode(i), x, y, z);
                if (tmp < minD) 
                {
                    minD = tmp;
                    minN = i;
                }
            }
    
            return SimNodePhysicalAddressOld.GetNode(minN);
        }

        // find the node closest to the given node
    /*  public SimNodePhysicalAddressOld[] GetSetNthClosest(NodePhysicalAddress node, int num) 
        {
            ArrayList minN = new ArrayList(num);
            ArrayList minD = new ArrayList(num);
        
            for (int i = 0; i < num; i++)
            {
                minD.Add(MaxDist);
                minN.Add(-1);               
            }
        
            if (SimNodePhysicalAddressOld.GetTotalNodes == 0) return null;
        
            for (int i=0; i<SimNodePhysicalAddressOld.GetTotalNodes; i++ ) 
            {
                int tmp = Distance(SimNodePhysicalAddressOld.GetNode(i), (SimNodePhysicalAddressOld)node);
                for (int j = 0; j < num; j++)
                {
                    if (tmp < (int) minD[j]) 
                    {
                        minN.Insert(j,i);
                        minD.Insert(j,tmp);
                        minN.RemoveAt(minN.Count-1);
                        minD.RemoveAt(minD.Count-1);
                        break;
                    }
                }
            }
    
            //return SimNodePhysicalAddressOld.GetNode((int) minN[num]);
            SimNodePhysicalAddressOld[] result = new SimNodePhysicalAddressOld[minN.Count];
            for (int i = 0; i < minN.Count; i++)
                result[i] = SimNodePhysicalAddressOld.GetNode((int) minN[i]);
            return result;
        }*/ 
    } 
}
