using System;

namespace Microsoft.Research.Herald.SkipNet {

    /// <summary>
    /// Various global Skip Net configuration Parameters.
    /// </summary>
    public class Parameters {

        //**********************************************************************************************
        // R-Table
        //**********************************************************************************************
        public static int   b=8;                        // The branching factor of ring-tree in the routing table
        public static int   maxHeight=32;               // The maximum height of the ring-tree in the routing table
        public static bool  denseRTable=true;           // Is the R-Table dense or sparse?
        public static int   leafSetHalfSize=8;          // The size of each half of the leaf set (bottom row of R-Table)
        public static bool  aggressiveFixups=false;     // If true, nodes do fixups more aggressively.
        
        //**********************************************************************************************
        // P-Table
        //**********************************************************************************************
        public static bool  proximityTableEnabled=true;     // Is the P Table enabled?
        public static int   maxPossibleEntries=10;          // The maximum number of possible entries
        public static bool  aggressivelyFindEntries=true;   // If true, nodes try to add entries from other nodes'
                                                            // join messages.
        public static int   maxRebuilds=0;                  // Maximum number of times to rebuild P-Table (0=infinity)
        public static int   pTableUpdatePeriod=10*60*1000;  // Rebuild P-table every 10 minutes
        
        //**********************************************************************************************
        // C-Table
        //**********************************************************************************************
        public static bool  cTableEnabled=true;             // Is the C Table enabled?
        public static bool  preferLongerPrefix=true;        // Are nodes with longer prefix match preferred?
        public static int   cTableUpdatePeriod=10*60*1000;  // Rebuild C-Table every 10 minutes
        
        //**********************************************************************************************
        // CLB-Table
        //**********************************************************************************************
        public static bool  clbTableEnabled=false;          // Is the CLB Table enabled?
        public static int   clbTableUpdatePeriod=2*60*1000; // Rebuild CLB Table every 2 minutes
        public static int   clbTableNbrListSize=2;          // The number of CLB neighbours to gather

        //**********************************************************************************************
        // Misc
        //**********************************************************************************************
        public static bool  recordMessagePath=false;        // Record the hops in each message
        
        //**********************************************************************************************
        // Periodic tasks
        //**********************************************************************************************
        public static bool  pingEnabled=false;
        public static bool  smartPingEnabled=true;          // If true, we batch pings between consecutive
        public static int   neighbourPingPeriod=1*60*1000;  // Ping every 1 minute
    
    
        //**********************************************************************************************
        // Private members
        //**********************************************************************************************
        private static int  seed;
    
    
        //**********************************************************************************************
        // Public Methods
        //**********************************************************************************************
        public static void SetSeed(int seed) {
            Parameters.seed = seed;
            NumericId.SetSeed(seed);
        }
    }
}
